<?php
/**
 * Deposits scheduled order manager
 *
 * @package woocommerce-deposits
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * WC_Deposits_Settings class.
 */
class WC_Deposits_Settings {

	/**
	 * Tab ID
	 *
	 * @var string
	 */
	private $settings_tab_id = 'deposits';

	/**
	 * Class instance
	 *
	 * @var WC_Deposits_Settings
	 */
	private static $instance;

	/**
	 * Get the class instance.
	 */
	public static function get_instance() {
		if ( null === self::$instance ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	/**
	 * Constructor.
	 */
	public function __construct() {
		// Load in the new settings tabs.
		add_action( 'woocommerce_get_sections_products', array( $this, 'add_woocommerce_settings_tab' ), 50 );
		add_action( 'woocommerce_get_settings_products', array( $this, 'get_settings' ), 50, 2 );

		add_action( 'admin_enqueue_scripts', array( $this, 'styles_and_scripts' ) );
	}

	/**
	 * Scripts.
	 */
	public function styles_and_scripts() {
		WC_Deposits::register_script( 'woocommerce-deposits-admin', 'admin' );
	}

	/**
	 * Add settings tab to woocommerce.
	 *
	 * @param array $settings_tabs Settings tabs.
	 * @return array
	 */
	public function add_woocommerce_settings_tab( $settings_tabs ) {
		$settings_tabs[ $this->settings_tab_id ] = __( 'Deposits', 'woocommerce-deposits' );
		return $settings_tabs;
	}

	/**
	 * Get the URL of the Deposits settings tab.
	 *
	 * @return string URL of the Deposits settings tab in WooCommerce settings.
	 */
	public function get_settings_tab_url(): string {
		return add_query_arg(
			array(
				'page'    => 'wc-settings',
				'tab'     => 'products',
				'section' => $this->settings_tab_id,
			),
			admin_url( 'admin.php' )
		);
	}

	/**
	 * Returns settings array.
	 *
	 * @param array  $settings Settings.
	 * @param string $current_section Section.
	 * @return array
	 */
	public function get_settings( $settings, $current_section ) {
		if ( 'deposits' !== $current_section ) {
			return $settings;
		}

		wp_enqueue_script( 'woocommerce-deposits-admin' );

		$payment_gateways        = WC()->payment_gateways->payment_gateways();
		$payment_gateway_options = array();

		foreach ( $payment_gateways as $gateway ) {
			$payment_gateway_options[ $gateway->id ] = $gateway->get_title();
		}

		$plans = WC_Deposits_Plans_Manager::get_plan_ids();

		$plan_ids = WC_Deposits_Plans_Manager::get_default_plan_ids();

		return apply_filters(
			'woocommerce_deposits_get_settings',
			array(
				array(
					'name' => __( 'Storewide Deposits Configuration', 'woocommerce-deposits' ),
					'type' => 'title',
					'desc' => __( 'These settings affect all products storewide. You can override these settings on a per product basis to make exceptions.', 'woocommerce-deposits' ),
					'id'   => 'deposits_defaults',
				),

				array(
					'name'     => __( 'Enable Deposits by Default', 'woocommerce-deposits' ),
					'type'     => 'select',
					'class'    => 'wc-enhanced-select',
					'desc'     => __( 'You must set a default amount below if setting this option to "yes".', 'woocommerce-deposits' ),
					'default'  => 'no',
					'id'       => 'wc_deposits_default_enabled',
					'desc_tip' => true,
					'options'  => array(
						'optional' => __( 'Yes - deposits are optional', 'woocommerce-deposits' ),
						'forced'   => __( 'Yes - deposits are required', 'woocommerce-deposits' ),
						'no'       => __( 'No', 'woocommerce-deposits' ),
					),
				),

				array(
					'name'     => esc_html__( 'Default Deposit Type', 'woocommerce-deposits' ),
					'type'     => 'select',
					'class'    => 'wc-enhanced-select',
					'desc'     => esc_html__( 'The default way for customers to pay for this product using a deposit.', 'woocommerce-deposits' ),
					'default'  => 'percent',
					'desc_tip' => true,
					'id'       => 'wc_deposits_default_type',
					'options'  => array(
						'none'    => esc_html__( 'None', 'woocommerce-deposits' ),
						'percent' => esc_html__( 'Percentage', 'woocommerce-deposits' ),
						'fixed'   => esc_html__( 'Fixed Amount', 'woocommerce-deposits' ),
						'plan'    => esc_html__( 'Payment Plan', 'woocommerce-deposits' ),
					),
				),

				array(
					'name'        => __( 'Default Deposit Amount', 'woocommerce-deposits' ),
					'type'        => 'text',
					'desc'        => __( 'The default deposit amount.', 'woocommerce-deposits' ),
					'default'     => '',
					'placeholder' => __( 'n/a', 'woocommerce-deposits' ),
					'id'          => 'wc_deposits_default_amount',
					'desc_tip'    => true,
				),

				array(
					'name'     => esc_html__( 'Default Deposit Selected Type', 'woocommerce-deposits' ),
					'type'     => 'select',
					'class'    => 'wc-enhanced-select',
					'desc'     => esc_html__( 'The default selected way for customers to pay for this product using a deposit.', 'woocommerce-deposits' ),
					'default'  => 'deposit',
					'desc_tip' => true,
					'id'       => 'wc_deposits_default_selected_type',
					'options'  => array(
						'deposit' => esc_html__( 'Pay Deposit', 'woocommerce-deposits' ),
						'full'    => esc_html__( 'Pay in Full', 'woocommerce-deposits' ),
					),
				),

				array(
					'name'              => esc_html__( 'Default Payment Plan', 'woocommerce-deposits' ),
					'type'              => 'multiselect',
					'class'             => 'wc-enhanced-select',
					'css'               => 'width: 450px;',
					'desc'              => esc_html__( 'The default payment plans to use.', 'woocommerce-deposits' ),
					'default'           => array(),
					'id'                => 'wc_deposits_default_plans',
					'custom_attributes' => array(
						'data-plans-order' => join( ',', $plan_ids ),
					),
					'desc_tip'          => true,
					'options'           => $plans,
				),

				array(
					'name'     => esc_html__( 'Disable Payment Gateways', 'woocommerce-deposits' ),
					'type'     => 'multiselect',
					'class'    => 'wc-enhanced-select',
					'css'      => 'width: 450px;',
					'desc'     => esc_html__( 'Select payment gateways that should be disabled when accepting deposits.', 'woocommerce-deposits' ),
					'default'  => '',
					'id'       => 'wc_deposits_disabled_gateways',
					'desc_tip' => true,
					'options'  => $payment_gateway_options,
				),

				array(
					'title'    => esc_html__( 'Order again behavior', 'woocommerce-deposits' ),
					'desc'     => esc_html__( 'Use original order\'s deposit settings on a reorder.', 'woocommerce-deposits' ),
					'desc_tip' => esc_html__( 'With this enabled, if a customer places a reorder, the same deposit settings as the original order will automatically be applied.', 'woocommerce-deposits' ),
					'id'       => 'wc_deposits_order_again_behaviour',
					'default'  => 'no',
					'type'     => 'checkbox',
				),

				array(
					'type' => 'sectionend',
					'id'   => 'deposits_defaults',
				),
			)
		);
	}
}

WC_Deposits_Settings::get_instance();
